<?php

if (!defined('_JEXEC'))
    die('Direct Access to ' . basename(__FILE__) . ' is not allowed.');

/**
 * Shipment plugin for Mondial relay
 *
 * @version $Id: mondial_relay.php
 * @package VirtueMart
 * @subpackage Plugins - shipment
 * @copyright Copyright (C) 2005-2011 Cg Consulting - All rights reserved.
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
 * VirtueMart is free software. This version may have been modified pursuant
 * to the GNU General Public License, and as distributed it includes or
 * is derivative of works licensed under the GNU General Public License or
 * other free or open source software licenses.
 * See /administrator/components/com_virtuemart/COPYRIGHT.php for copyright notices and details.
 *
 * http://cg-consulting.fr
 * @author Cyrille Guillaud
 *
 */
if (!class_exists('vmPSPlugin'))
    require(JPATH_VM_PLUGINS . DS . 'vmpsplugin.php');

class plgVmShipmentMondial_relay extends vmPSPlugin {

    // instance of class
    public static $_this = false;
    private $_point_num = '';
    private $_price_table = array();

	private $_zipcode='';
	
    function __construct(& $subject, $config) {

	parent::__construct($subject, $config);

	$this->_loggable = true;
	$this->tableFields = array_keys($this->getTableSQLFields());
	$varsToPush = $this->getVarsToPush();
	$this->setConfigParameterable($this->_configTableFieldName, $varsToPush);

    }

    /**
     * Create the table for this plugin if it does not yet exist.
     */
    public function getVmPluginCreateTableSQL() {

	return $this->createTableSQL('Shipment Mondial Relay Table');
    }

    function getTableSQLFields() {
	$SQLfields = array(
	    'id' => ' int(1) unsigned NOT NULL AUTO_INCREMENT',
	    'virtuemart_order_id' => 'int(11) UNSIGNED DEFAULT NULL',
	    'order_number' => 'char(32) DEFAULT NULL',
	    'virtuemart_shipmentmethod_id' => 'mediumint(1) UNSIGNED DEFAULT NULL',
	    'shipment_name' => 'varchar(5000)',
	    'shipment_point_num' => 'varchar(256)',
	    'shipment_tracking' => 'varchar(256)',
	    'order_weight' => 'decimal(10,4) DEFAULT NULL',
	    'shipment_weight_unit' => 'char(3) DEFAULT \'KG\' ',
	    'shipment_cost' => 'decimal(10,2) DEFAULT NULL',
	    'shipment_package_fee' => 'decimal(10,2) DEFAULT NULL',
	    'tax_id' => 'smallint(1) DEFAULT NULL'
	);
	return $SQLfields;
    }

    /**
     * This method is fired when showing the order details in the frontend.
     * It displays the shipment-specific data.
     *
     * @param integer $order_number The order Number
     * @return mixed Null for shipments that aren't active, text (HTML) otherwise
     * @author Valérie Isaksen
     * @author Max Milbers
     */
    public function plgVmOnShowOrderFEShipment($virtuemart_order_id, $virtuemart_shipmentmethod_id, &$shipment_name) {
	$this->onShowOrderFE($virtuemart_order_id, $virtuemart_shipmentmethod_id, $shipment_name);
    }

    /**
     * This event is fired after the order has been stored; it gets the shipment method-
     * specific data.
     *
     * @param int $order_id The order_id being processed
     * @param object $cart  the cart
     * @param array $priceData Price information for this order
     * @return mixed Null when this method was not selected, otherwise true
     * @author Valerie Isaksen
     */
    function plgVmConfirmedOrder(VirtueMartCart $cart, $order) {
	if (!($method = $this->getVmPluginMethod($order['details']['BT']->virtuemart_shipmentmethod_id))) {
	    return null; // Another method was selected, do nothing
	}
	if (!$this->selectedThisElement($method->shipment_element)) {
	    return false;
	}
	$values['virtuemart_order_id'] = $order['details']['BT']->virtuemart_order_id;
	$values['order_number'] = $order['details']['BT']->order_number;
	$values['virtuemart_shipmentmethod_id'] = $order['details']['BT']->virtuemart_shipmentmethod_id;
	
	//Modif d'Encros
	//$orderWeight = ($this->getOrderWeight($cart, $method->weight_unit) < 1) ? 1 : $this->getOrderWeight($cart, $method->weight_unit) ;
	$orderWeight = ($this->getOrderWeight($cart, $method->weight_unit) < 500) ? 500 : $this->getOrderWeight($cart, $method->weight_unit) ;
	
	$this->_getMondialrelayIntoSession();
	$relais_id = $this->_point_num;
	vmdebug('point_num : '.$relais_id);
	$values['shipment_point_num'] = $this->_point_num;
	$values['shipment_name'] = $this->renderPluginName($method);
	//$values['order_weight'] = $this->getOrderWeight($cart, $method->weight_unit);
	//Modif d'Encros
	$values['order_weight'] = $orderWeight;
	$values['shipment_weight_unit'] = $method->weight_unit;
	$values['shipment_cost'] = $method->cost;
	$values['shipment_package_fee'] = $method->package_fee;
	$values['tax_id'] = $method->tax_id;
	$this->storePSPluginInternalData($values);

	return true;
    }

    /**
     * This method is fired when showing the order details in the backend.
     * It displays the shipment-specific data.
     * NOTE, this plugin should NOT be used to display form fields, since it's called outside
     * a form! Use plgVmOnUpdateOrderBE() instead!
     *
     * @param integer $virtuemart_order_id The order ID
     * @param integer $vendorId Vendor ID
     * @param object $_shipInfo Object with the properties 'shipment' and 'name'
     * @return mixed Null for shipments that aren't active, text (HTML) otherwise
     * @author Valerie Isaksen
     */
    public function plgVmOnShowOrderBEShipment($virtuemart_order_id, $virtuemart_shipmentmethod_id) {

	if (!($this->selectedThisByMethodId($virtuemart_shipmentmethod_id))) {
	    return null;
	}
	$html = $this->getOrderShipmentHtml($virtuemart_order_id);
	return $html;
    }



    public function plgVmonShowStickerPrintPayment($virtuemart_order_id, $virtuemart_shipmentmethod_id) {

		$db = JFactory::getDBO();
	$q = 'SELECT `shipment_point_num`,`shipment_tracking` FROM `' . $this->_tablename . '` '
		. 'WHERE `virtuemart_order_id` = ' . $virtuemart_order_id;
	$db->setQuery($q);
	$shipinfo = $db->loadObject();
	if (!($shipinfo)) {
	    //vmWarn(500, $q . " " . $db->getErrorMsg());
	    return false;
	}

	return $shipinfo;
    }
    
    function getOrderShipmentHtml($virtuemart_order_id) {

	$db = JFactory::getDBO();
	$q = 'SELECT * FROM `' . $this->_tablename . '` '
		. 'WHERE `virtuemart_order_id` = ' . $virtuemart_order_id;
	$db->setQuery($q);
	if (!($shipinfo = $db->loadObject())) {
	    vmWarn(500, $q . " " . $db->getErrorMsg());
	    return '';
	}

	if (!class_exists('CurrencyDisplay'))
	    require(JPATH_VM_ADMINISTRATOR . DS . 'helpers' . DS . 'currencydisplay.php');

	$currency = CurrencyDisplay::getInstance();
	$tax = ShopFunctions::getTaxByID($shipinfo->tax_id);
	$taxDisplay = is_array($tax) ? $tax['calc_value'] . ' ' . $tax['calc_value_mathop'] : $shipinfo->tax_id;
	$taxDisplay = ($taxDisplay == -1 ) ? JText::_('COM_VIRTUEMART_PRODUCT_TAX_NONE') : $taxDisplay;

	$html = '<table class="adminlist">' . "\n";
	$html .=$this->getHtmlHeaderBE();
	$html .= $this->getHtmlRowBE('MR_COUNTRIES_SHIPPING_NAME', $shipinfo->shipment_name);
	$html .= $this->getHtmlRowBE('MR_COUNTRIES_WEIGHT', $shipinfo->order_weight . ' ' . ShopFunctions::renderWeightUnit($shipinfo->shipment_weight_unit));
	$html .= $this->getHtmlRowBE('MR_COUNTRIES_COST', $currency->priceDisplay($shipinfo->shipment_cost, '', false));
	$html .= $this->getHtmlRowBE('MR_COUNTRIES_PACKAGE_FEE', $currency->priceDisplay($shipinfo->shipment_package_fee, '', false));
	$html .= $this->getHtmlRowBE('MR_COUNTRIES_TAX', $taxDisplay);
	$html .= '</table>' . "\n";

	return $html;
    }

    function getCosts(VirtueMartCart $cart, $method, $cart_prices) {

	if ($method->free_shipment && $cart_prices['salesPrice'] >= $method->free_shipment) {
	    return 0;
	} else {
		$orderWeight = $this->getOrderWeight($cart, $method->weight_unit);
		$cost = $this->plgVmGetWeightPrice($method,$orderWeight);

	    return $cost[1] + $method->package_fee;
	}
    }
    
    function plgVmGetWeightPrice($method,$orderWeight){
		$priceToReturn = array();

		if(!count($this->_price_table) &&$method->cost){
			$prices = explode(',',trim($method->cost));
			foreach($prices as $price){
				$weight_price = explode(':',trim($price));
				$this->_price_table[$weight_price[0]] = $weight_price[1];
			}
		}
		
		foreach($this->_price_table as $weight=>$price){

				if($orderWeight <= $weight){
    				if(!count($priceToReturn)){

						$priceToReturn[0] = $weight;
						$priceToReturn[1] = $price;
    				}else{
    					if($weight <= $priceToReturn[0]){
							$priceToReturn[0] = $weight;
							$priceToReturn[1] = $price;
    					}
    				}
				}
		}

		return $priceToReturn;

    }

    protected function checkConditions($cart, $method, $cart_prices) {


	$orderWeight = $this->getOrderWeight($cart, $method->weight_unit);
	$address = (($cart->ST == 0) ? $cart->BT : $cart->ST);

	$nbShipment = 0;
	$countries = array();
	if (!empty($method->countries)) {
	    if (!is_array($method->countries)) {
		$countries[0] = $method->countries;
	    } else {
		$countries = $method->countries;
	    }
	}
	// probably did not gave his BT:ST address
	if (!is_array($address)) {
	    $address = array();
	    $address['zip'] = 0;
	    $address['virtuemart_country_id'] = 0;
	}

	$weight_cond = count($this->plgVmGetWeightPrice($method,$orderWeight));
	$orderamount_cond = $this->_orderamountCond($cart_prices, $method);
	if (isset($address['zip'])) {
	    $zip_cond = $this->_zipCond($address['zip'], $method);
	} else {
	    //no zip in address data normally occurs only, when it is removed from the form by the shopowner
	    //Todo for  valerie, you may take a look, maybe should be false, or configurable.
	    $zip_cond = true;
	}

	$this->_zipcode=$address['zip'];  
	if (!isset($address['virtuemart_country_id']))
	    $address['virtuemart_country_id'] = 0;
	if (in_array($address['virtuemart_country_id'], $countries) || count($countries) == 0) {
	    if ($weight_cond AND $zip_cond AND $orderamount_cond) {
		return true;
	    }
	}

	return false;
    }


     private function _orderamountCond($cart_prices, $method) {
	$orderamount = 0;

	if (!isset($method->orderamount_start) AND !isset($method->orderamount_stop)) {
	    return true;
	}
	if ($cart_prices['salesPrice']) {
	    $orderamount_cond = ($cart_prices['salesPrice'] >= $method->orderamount_start AND $cart_prices['salesPrice'] <= $method->orderamount_stop
		    OR
		    ($method->orderamount_start <= $cart_prices['salesPrice'] AND ($method->orderamount_stop == 0) ));
	} else {
	    $orderamount_cond = true;
	}
	return $orderamount_cond;
    }
    /**
     * Check the conditions on Zip code
     * @param int $zip : zip code
     * @param $params paremters for this specific shiper
     * @author Valérie Isaksen
     * @return string if Zip condition is ok or not
     */
    private function _zipCond($zip, $method) {
	if (!empty($zip)) {
	    $zip_cond = (( $zip >= $method->zip_start AND $zip <= $method->zip_stop )
		    OR
		    ($method->zip_start <= $zip AND ($method->zip_stop == 0) ));
	} else {
	    $zip_cond = true;
	}
	return $zip_cond;
    }

    /*
     * We must reimplement this triggers for joomla 1.7
     */

    /**
     * Create the table for this plugin if it does not yet exist.
     * This functions checks if the called plugin is active one.
     * When yes it is calling the standard method to create the tables
     * @author Valérie Isaksen
     *
     */
    function plgVmOnStoreInstallShipmentPluginTable($jplugin_id) {
	return $this->onStoreInstallPluginTable($jplugin_id);
    }

    /**
     * This event is fired after the shipment method has been selected. It can be used to store
     * additional payment info in the cart.
     *
     * @author Max Milbers
     * @author Valérie isaksen
     *
     * @param VirtueMartCart $cart: the actual cart
     * @return null if the payment was not selected, true if the data is valid, error message if the data is not vlaid
     *
     */
    public function plgVmOnSelectCheck($psType, VirtueMartCart $cart) {
	return $this->OnSelectCheck($psType, $cart);
    }

    /**
     * plgVmDisplayListFE
     * This event is fired to display the pluginmethods in the cart (edit shipment/payment) for exampel
     *
     * @param object $cart Cart object
     * @param integer $selected ID of the method selected
     * @return boolean True on succes, false on failures, null when this plugin was not selected.
     * On errors, JError::raiseWarning (or JError::raiseError) must be used to set a message.
     *
     * @author Valerie Isaksen
     * @author Max Milbers
     */
    public function plgVmDisplayListFEShipment(VirtueMartCart $cart, $selected = 0, &$htmlIn) {
		return $this->displayListFE($cart, $selected, $htmlIn);
    }
	
	/**
	 * 2
	 * This event is fired to display the pluginmethods in the cart (edit shipment/payment) for exampel
	 *
	 * @param object $cart Cart object
	 * @param integer $selected ID of the method selected
	 * @return boolean True on succes, false on failures, null when this plugin was not selected.
	 * On errors, JError::raiseWarning (or JError::raiseError) must be used to set a message.
	 *
	 * @author Valerie Isaksen
	 * @author Max Milbers
	 */
	public function displayListFE(VirtueMartCart $cart, $selected = 0, &$htmlIn) {

		if ($this->getPluginMethods($cart->vendorId) === 0) {
			if (empty($this->_name)) {
				$app = JFactory::getApplication();
				$app->enqueueMessage(JText::_('COM_VIRTUEMART_CART_NO_' . strtoupper($this->_psType)));
				return false;
			} else {
				return false;
			}
		}

		$html = array();
		$method_name = $this->_psType . '_name';
		foreach ($this->methods as $method) {
			if ($this->checkConditions($cart, $method, $cart->pricesUnformatted)) {
				$methodSalesPrice = $this->calculateSalesPrice($cart, $method, $cart->pricesUnformatted);
				$method->$method_name = $this->renderPluginName($method);
				$html [] = $this->getPluginHtml($method, $selected, $methodSalesPrice);
			}
		}
		$htmlIn[] = $html;

		return true;
	}
	
	public function onSelectedCalculatePrice(VirtueMartCart $cart, array &$cart_prices, &$cart_prices_name) {
		$id = $this->_idName;
		if (!($method = $this->selectedThisByMethodId($cart->$id))) {
			return null; // Another method was selected, do nothing
		}

		if (!($method = $this->getVmPluginMethod($cart->$id) )) {
			return null;
		}

		$cart_prices_name = '';
		$cart_prices[$this->_psType . '_tax_id'] = 0;
		$cart_prices['cost'] = 0;

		if (!$this->checkConditions($cart, $method, $cart_prices)) {
			return false;
		}
		$paramsName = $this->_psType . '_params';
		
		$cart_prices_name = $this->renderPluginName($method);
		$this->setCartPrices($cart, $cart_prices, $method);

		return true;
	}
	
	public function plgVmOnSelectCheckShipment(VirtueMartCart $cart) {
        vmdebug('test2');
		$idName = $this->_idName;
		
		//$cart->setCartIntoSession();
		if (!$this->selectedThisByMethodId($cart->$idName)) {
			return null; // Another method was selected, do nothing
		}
		$virtuemart_shipmentmethod_id = JRequest::getString('virtuemart_shipmentmethod_id');
		$param = explode('_',$virtuemart_shipmentmethod_id);
		if(isset($param[1]) && trim($param[1]) != ''){
			vmdebug('OnSelectCheck',$param[1]);
			//$cart->setShipment($virtuemart_shipmentmethod_id);
			$this->_point_num = $param[1];
			$this->_setMondialrelayIntoSession();
		}
		return true; // this method was selected , and the data is valid by default
	}
	
	
	protected function _setMondialrelayIntoSession() {
		$session = JFactory::getSession();
		$sessionMondialrelay = new stdClass();
		// card information
		$sessionMondialrelay->point_num = $this->_point_num;
		$session->set('mondialrelay', serialize($sessionMondialrelay), 'vm');
    }

    protected function _getMondialrelayIntoSession() {
		$session = JFactory::getSession();
		$sessionMondialrelay = $session->get('mondialrelay', 0, 'vm');
	
		if (!empty($sessionMondialrelay)) {
			$mondialrelayData = unserialize($sessionMondialrelay);
			$this->_point_num = $mondialrelayData->point_num;
		}
    }
	
	protected function getPluginHtml($plugin, $selectedPlugin, $pluginSalesPrice) {
		vmdebug('test');
		//echo 'getPluginHtml';
		//echo $plugin->enseigne;
		//print_r($plugin);
		
		$pluginmethod_id = $this->_idName;
		$pluginName = $this->_psType . '_name';
		if ($selectedPlugin == $plugin->$pluginmethod_id) {
			$checked = 'checked';
		} else {
			$checked = '';
		}

		if (!class_exists('CurrencyDisplay'))
		require(JPATH_VM_ADMINISTRATOR . DS . 'helpers' . DS . 'currencydisplay.php');
		$currency = CurrencyDisplay::getInstance();
		$costDisplay="";
		if ($pluginSalesPrice) {
			$costDisplay = $currency->priceDisplay($pluginSalesPrice);
			$costDisplay ='<span class="' . $this->_type . '_cost"> ' . JText::_('COM_VIRTUEMART_PLUGIN_COST_DISPLAY').$costDisplay . "</span>";
		}

// On met en place les paramÃ¨tres de la requÃ¨te
		$params = array(
					   'Enseigne'     => $plugin->enseigne,
					   'Pays'         => 'FR',

					   'CP'           => $this->_zipcode, 
		);
		
		//On crÃ©e le code de sÃ©curitÃ©
		$code = implode("",$params);
		$code .= $plugin->cle;
				
		//On le rajoute aux paramÃ¨tres
		$params["Security"] = strtoupper(md5($code));
				
		// On se connecte
		$client = new SoapClient("http://www.mondialrelay.fr/WebService/Web_Services.asmx?WSDL");
				
		// Et on effectue la requÃ¨te
		$points_relais = $client->WSI2_RecherchePointRelais($params)->WSI2_RecherchePointRelaisResult;
		//print_r($points_relais);
		
	
		if( count($points_relais)){
			
			$html = '<div id="mondial_relay"></div><div width="100%">Ou dans un point retrait :<br>Avec Mondial Relay :</div>' ;
			//$html .= '' . "</th><th>" ;
			//$html .=  "</th></tr>\n" ;
			$html .='<div style="font-weight:bold;"><img align="middle" src="http://www.votre-site.fr/images/stories/virtuemart/shipment/mondialrelay.png" alt="9">&nbsp;A compter de la prise en charge par Mondial Relay, vous &ecirc;tes livr&eacute; entre 3 &agrave; 7 jours.<br><br>Choisissez le point Mondial Relay le plus proche de chez vous dans la liste ci-dessous :<br>(Cliquez sur "Détails" pour voir le plan et les horaires...)</div>';
		
		}
		
		$this->_getMondialrelayIntoSession();
		$relais_id = $this->_point_num;
		
		//while( $dbr->next_record() ) {
			foreach( $points_relais as $point_relais ) {
				//echo 'test:'.count($points_relais);
				if ( is_object($point_relais) && trim($point_relais->Num) != '' ) {		
					$checked = '';
					if($this->selectedThisByMethodId($cart->$idName) && $point_relais->Num == $relais_id)	{
						$checked = 'checked';
					}		
					//$html .= '<tr style="border:none;">' ;
	$popup_params = array(
								   'ens'     => $plugin->marque_url,
								   'num'     => $point_relais->Num,
								   'pays'    => $point_relais->Pays,
					);
					
					
					$popup_code = '<'.$plugin->marque_url.'>'.$point_relais->Num.$point_relais->Pays.'<'.$plugin->cle.'>';
							
					
					$popup_params["Security"] = strtoupper(md5($popup_code));
					
					$link = 'http://www.mondialrelay.com/lg_fr/espaces/url/popup_pointrelais.aspx?ens='. $popup_params["ens"] .'&num='.$popup_params["num"].'&pays='.$popup_params["pays"].'&crc='.$popup_params["Security"];
					
					$html .= '<div style="border:none;" ><input type="radio" class="mr" name="' . $pluginmethod_id . '" id="' . $this->_psType . '_id_' . $plugin->$pluginmethod_id . '"   value="' . $plugin->$pluginmethod_id.'_' .$point_relais->Num. '" ' . $checked . ">\n"
		. '<label for="' . $this->_psType . '_id_' . $plugin->$pluginmethod_id . '">' . '<span class="' . $this->_type . '"> ' . ' - ' . $point_relais->LgAdr1 . ' - ' . $point_relais->Ville  . ' '.shopFunctionsF::vmPopupLink( $link, 'D&eacute;tails', '800', '450' )."</span></label></td>";
					
				
					$html .= '<div style="border:none;">' . $costDisplay . "</div>\n" ;
				}
			}
		//}
		if(count($points_relais)){
			//$html .= "</table>\n" ;
		}
		
		return $html;
	}
	

    /*
     * plgVmonSelectedCalculatePrice
     * Calculate the price (value, tax_id) of the selected method
     * It is called by the calculator
     * This function does NOT to be reimplemented. If not reimplemented, then the default values from this function are taken.
     * @author Valerie Isaksen
     * @cart: VirtueMartCart the current cart
     * @cart_prices: array the new cart prices
     * @return null if the method was not selected, false if the shiiping rate is not valid any more, true otherwise
     *
     *
     */

    public function plgVmonSelectedCalculatePriceShipment(VirtueMartCart $cart, array &$cart_prices, &$cart_prices_name) {
	return $this->onSelectedCalculatePrice($cart, $cart_prices, $cart_prices_name);
    }

    /**
     * plgVmOnCheckAutomaticSelected
     * Checks how many plugins are available. If only one, the user will not have the choice. Enter edit_xxx page
     * The plugin must check first if it is the correct type
     * @author Valerie Isaksen
     * @param VirtueMartCart cart: the cart object
     * @return null if no plugin was found, 0 if more then one plugin was found,  virtuemart_xxx_id if only one plugin is found
     *
     */
    function plgVmOnCheckAutomaticSelectedShipment(VirtueMartCart $cart, array $cart_prices = array()) {
    	//return null to force the selection
    	
//    	$check = $this->onCheckAutomaticSelected($cart, $cart_prices);

		return null;
    }
    

    /**
     * This event is fired during the checkout process. It can be used to validate the
     * method data as entered by the user.
     *
     * @return boolean True when the data was valid, false otherwise. If the plugin is not activated, it should return null.
     * @author Max Milbers

      public function plgVmOnCheckoutCheckData($psType, VirtueMartCart $cart) {
      return null;
      }
     */

    /**
     * This method is fired when showing when priting an Order
     * It displays the the payment method-specific data.
     *
     * @param integer $_virtuemart_order_id The order ID
     * @param integer $method_id  method used for this order
     * @return mixed Null when for payment methods that were not selected, text (HTML) otherwise
     * @author Valerie Isaksen
     */
    function plgVmonShowOrderPrint($order_number, $method_id) {
	return $this->onShowOrderPrint($order_number, $method_id);
    }

    /**
     * Save updated order data to the method specific table
     *
     * @param array $_formData Form data
     * @return mixed, True on success, false on failures (the rest of the save-process will be
     * skipped!), or null when this method is not actived.
     * @author Oscar van Eijk

      public function plgVmOnUpdateOrder($psType, $_formData) {
      return null;
      }
     */
    /**
     * Save updated orderline data to the method specific table
     *
     * @param array $_formData Form data
     * @return mixed, True on success, false on failures (the rest of the save-process will be
     * skipped!), or null when this method is not actived.
     * @author Oscar van Eijk

      public function plgVmOnUpdateOrderLine($psType, $_formData) {
      return null;
      }
     */
    /**
     * plgVmOnEditOrderLineBE
     * This method is fired when editing the order line details in the backend.
     * It can be used to add line specific package codes
     *
     * @param integer $_orderId The order ID
     * @param integer $_lineId
     * @return mixed Null for method that aren't active, text (HTML) otherwise
     * @author Oscar van Eijk

      public function plgVmOnEditOrderLineBE($psType, $_orderId, $_lineId) {
      return null;
      }
     */
    /**
     * This method is fired when showing the order details in the frontend, for every orderline.
     * It can be used to display line specific package codes, e.g. with a link to external tracking and
     * tracing systems
     *
     * @param integer $_orderId The order ID
     * @param integer $_lineId
     * @return mixed Null for method that aren't active, text (HTML) otherwise
     * @author Oscar van Eijk

      public function plgVmOnShowOrderLineFE($psType, $_orderId, $_lineId) {
      return null;
      }
     */

    /**
     * plgVmOnResponseReceived
     * This event is fired when the  method returns to the shop after the transaction
     *
     *  the method itself should send in the URL the parameters needed
     * NOTE for Plugin developers:
     *  If the plugin is NOT actually executed (not the selected payment method), this method must return NULL
     *
     * @param int $virtuemart_order_id : should return the virtuemart_order_id
     * @param text $html: the html to display
     * @return mixed Null when this method was not selected, otherwise the true or false
     *
     * @author Valerie Isaksen
     *

      function plgVmOnResponseReceived($psType, &$virtuemart_order_id, &$html) {
      return null;
      }
     */
    function plgVmDeclarePluginParamsShipment($name, $id, &$data) {

	return $this->declarePluginParams('shipment', $name, $id, $data);
    }

    function plgVmSetOnTablePluginParamsShipment($name, $id, &$table) {
	return $this->setOnTablePluginParams($name, $id, $table);
    }
    
    
    public function plgVmRegisterShipping($virtuemart_order_id)
	{

		$orderModel = VmModel::getModel('orders');
		$order = $orderModel->getOrder($virtuemart_order_id);

		$db = JFactory::getDBO();
		$q = 'SELECT * FROM `' . $this->_tablename . '` '
			. 'WHERE `virtuemart_order_id` = ' . $virtuemart_order_id;
		$db->setQuery($q);
		$shipinfo = $db->loadObject();
		if (!($shipinfo)) {
		    vmWarn(500, $q . " " . $db->getErrorMsg());
		    return null;
		}
		
		$shipment_point_num = $shipinfo->shipment_point_num;
		$virtuemart_shipment_id = $shipinfo->id;
		$coef = 1000;
		if(trim($shipinfo->shipment_weight_unit) == 'G') $coef = 1;
		$weight = ((int)$shipinfo->order_weight)*((int)$coef);

		$query = 'SELECT *';
		$query .= ' FROM `#__virtuemart_countries`';
		$query .= ' WHERE `virtuemart_country_id` = ' . (int)$order['details']['BT']->virtuemart_country_id;
		$db->setQuery($query);
		$countryinfo = $db->loadObject();
		if (!($countryinfo)) {
		    vmWarn(500, $q . " " . $db->getErrorMsg());
		    return null;
		}
		$dest_country = $countryinfo->country_2_code;

//echo $shipment_point_num;
		echo $dest_country;
//		die;
		/* Read current Configuration */
		if (!($method = $this->getVmPluginMethod($order['details']['BT']->virtuemart_shipmentmethod_id))) {
		    return null; // Another method was selected, do nothing
		}

		// On met en place les paramètres de la requète
// On met en place les paramètres de la requète
		$params = array(
					   'Enseigne'  => $method->enseigne,
					   'Num'       => $shipment_point_num,
					   'Pays'      => $dest_country
		);

		//On crée le code de sécurité
		$code = implode("",$params);
		$code .= $method->cle;
		

		//On le rajoute aux paramètres
		$params["Security"] = strtoupper(md5($code));
		//echo 'Security: '.$params["Security"];
		// On se connecte
		$client = new SoapClient("http://www.mondialrelay.fr/WebService/Web_Services.asmx?WSDL");
		
		// Et on effectue la requète
		$detail_pointrelais = $client->WSI2_DetailPointRelais($params)->WSI2_DetailPointRelaisResult;
		
		//echo $shipment_point_num;
  


		if($detail_pointrelais->STAT == '0'){

			// create request

        
 
	        $params = array(
	                        'Enseigne'       => trim($method->enseigne),
	                        'ModeCol'        => 'CCC',
	                        'ModeLiv'        => '24R',
	                        'Expe_Langage'   => 'FR',
	                        'Expe_Ad1'       => trim($this->removeaccents($method->address1)),
	                        'Expe_Ad3'       => trim($this->removeaccents($method->address3)),
	                        'Expe_Ad4'       => trim($this->removeaccents($method->address4)),
	                        'Expe_Ville'     => trim($this->removeaccents($method->ville)),
	                        'Expe_CP'        => trim($method->enseigne_cp),
	                        'Expe_Pays'      => trim($this->removeaccents($method->pays)),
	                        'Expe_Tel1'      => trim($method->tel),
	                        'Expe_Tel2'      => trim($method->mobile),
	                        'Expe_Mail'      => trim($method->mail),
	                        'Dest_Langage'   => 'FR',
	                        'Dest_Ad1'       => trim($this->removeaccents($order['details']['BT']->first_name . ' ' . $order['details']['BT']->last_name)),
	                        'Dest_Ad2'       => trim($this->removeaccents($detail_pointrelais->LgAdr2)),
	                        'Dest_Ad3'       => trim($this->removeaccents($detail_pointrelais->LgAdr3)),
	                        'Dest_Ad4'       => trim($this->removeaccents($detail_pointrelais->LgAdr4)),                                   
	                        'Dest_Ville'     => trim($this->removeaccents($detail_pointrelais->Ville)),
	                        'Dest_CP'        => trim($this->removeaccents($detail_pointrelais->CP)),
	                        'Dest_Pays'      => trim($this->removeaccents($detail_pointrelais->Pays)),
	                        'Dest_Tel1'      => trim($this->removeaccents($order['details']['BT']->phone_1)),
	                        'Dest_Mail'      => trim($order['details']['BT']->email),
	                        'Poids'          => $weight,
	                        'NbColis'        => '1',
	                        'CRT_Valeur'     => '0',
	                        'LIV_Rel_Pays'   => $dest_country,
	                        'LIV_Rel'        => $shipment_point_num,
	            );
    

	        $code = implode("",$params);
	        $code .= $method->cle;

	        //On le rajoute aux paramètres
	        $params["Security"] = strtoupper(md5($code));
	                    
	        // On se connecte
	        $client = new SoapClient("http://www.mondialrelay.fr/WebService/Web_Services.asmx?WSDL");
	                    
	        //send request
	        $expedition = $client->WSI2_CreationExpedition($params)->WSI2_CreationExpeditionResult;
	        
//echo $expedition->STAT;
//print_r($shipinfo);
//die;
			if($expedition->STAT == '0'){
				$values = array();
				foreach($shipinfo as $key=>$value){
					$values[$key] = $value;
					echo $key.' : '.$value;
				}
				$values['shipment_tracking'] = $expedition->ExpeditionNum;
				$this->storePSPluginInternalData($values);
			}else{
				vmdebug('Impossible de creer l\'expedition');
				vmdebug('erreur: '.$expedition->STAT);
			}
		}else{
			vmdebug('Impossible de se connecter a mondialrelay');
			vmdebug('erreur: '.$detail_pointrelais->STAT);
		}
		return (true);
	}
	
	 public function plgVmprintShipping($virtuemart_order_id)
	{

		$orderModel = VmModel::getModel('orders');
		$order = $orderModel->getOrder($virtuemart_order_id);

		$db = JFactory::getDBO();
		$q = 'SELECT * FROM `' . $this->_tablename . '` '
			. 'WHERE `virtuemart_order_id` = ' . $virtuemart_order_id;
		$db->setQuery($q);
		$shipinfo = $db->loadObject();
		if (!($shipinfo)) {
		    vmWarn(500, $q . " " . $db->getErrorMsg());
		    return null;
		}
		
		$tracking_number = $shipinfo->shipment_tracking;
		$virtuemart_shipment_id = $shipinfo->id;


		$query = 'SELECT *';
		$query .= ' FROM `#__virtuemart_countries`';
		$query .= ' WHERE `virtuemart_country_id` = ' . (int)$order['details']['BT']->virtuemart_country_id;
		$db->setQuery($query);
		$countryinfo = $db->loadObject();
		if (!($countryinfo)) {
		    vmWarn(500, $q . " " . $db->getErrorMsg());
		    return null;
		}
		$dest_country = $countryinfo->country_2_code;
		if (!($method = $this->getVmPluginMethod($order['details']['BT']->virtuemart_shipmentmethod_id))) {
		    return null; // Another method was selected, do nothing
		}
		// On met en place les param\E8tres de la requ\E8te
        $params = array(
                       'Enseigne'       => trim($method->enseigne),
                       'Expeditions'    => $tracking_number,
                       'Langue'    		=> $dest_country,
        );
        //On cr\E9e le code de s\E9curit\E9
        $code = implode("",$params);
        $code .= trim($method->cle);
        
        //On le rajoute aux param\E8tres
        $params["Security"] = strtoupper(md5($code));
        
        // On se connecte
        $client = new SoapClient("http://www.mondialrelay.fr/WebService/Web_Services.asmx?WSDL");
	    // Et on effectue la requ\E8te
	    $etiquette = $client->WSI2_GetEtiquettes($params)->WSI2_GetEtiquettesResult;
        

        if($etiquette->STAT == '0'){
//			echo 'http://www.mondialrelay.fr' . $etiquette->URL_PDF_A5;
//			die;
      	 	return 'http://www.mondialrelay.fr' . $etiquette->URL_PDF_A5;
		}else{
			vmdebug('Impossible de creer l\'etiquette');
			vmdebug('erreur: '.$etiquette->STAT);
		}
		return false;
	}

    function removeaccents($string){ 
		return $this->cleanText($string);
   } 
   
   		function cleanText($toClean) {
		    $table = array(
		        '?'=>'S', '?'=>'s', '?'=>'Dj', '?'=>'dj', '?'=>'Z', '?'=>'z', '?'=>'C', '?'=>'c', '?'=>'C', '?'=>'c',






		        '\C0'=>'A', '\C1'=>'A', '\C2'=>'A', '\C3'=>'A', '\C4'=>'A', '\C5'=>'A', '\C6'=>'A', '\C7'=>'C', '\C8'=>'E', '\C9'=>'E',
		        '\CA'=>'E', '\CB'=>'E', '\CC'=>'I', '\CD'=>'I', '\CE'=>'I', '\CF'=>'I', '\D1'=>'N', '\D2'=>'O', '\D3'=>'O', '\D4'=>'O',
		        '\D5'=>'O', '\D6'=>'O', '\D8'=>'O', '\D9'=>'U', '\DA'=>'U', '\DB'=>'U', '\DC'=>'U', '\DD'=>'Y', '\DE'=>'B', '\DF'=>'Ss',
		        '\E0'=>'a', '\E1'=>'a', '\E2'=>'a', '\E3'=>'a', '\E4'=>'a', '\E5'=>'a', '\E6'=>'a', '\E7'=>'c', '\E8'=>'e', '\E9'=>'e',
		        '\EA'=>'e', '\EB'=>'e', '\EC'=>'i', '\ED'=>'i', '\EE'=>'i', '\EF'=>'i', '\F0'=>'o', '\F1'=>'n', '\F2'=>'o', '\F3'=>'o',
		        '\F4'=>'o', '\F5'=>'o', '\F6'=>'o', '\F8'=>'o', '\F9'=>'u', '\FA'=>'u', '\FB'=>'u', '\FD'=>'y', '\FD'=>'y', '\FE'=>'b',
		        '\FF'=>'y', '?'=>'R', '?'=>'r',
		    );

			$toCleanNOUTF8 = utf8_decode($toClean);

			$string = strtr($toCleanNOUTF8, $table);
			$toCleanUTF8 = utf8_encode($string);

		    return $toCleanUTF8;
		}
		
}

// No closing tag